import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/common/dialog_dropdown.dart';
import 'package:syncrow_web/common/dialog_textfield_dropdown.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/dialog/create_space_model_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/tag_model/views/add_device_type_model_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/pages/spaces_management/helper/tag_helper.dart';

class AssignTagModelsDialog extends StatelessWidget {
  final List<ProductModel>? products;
  final List<SubspaceTemplateModel>? subspaces;
  final SpaceTemplateModel? spaceModel;

  final List<TagModel> initialTags;
  final ValueChanged<List<TagModel>>? onTagsAssigned;
  final List<SelectedProduct> addedProducts;
  final List<String>? allTags;
  final String spaceName;
  final String title;
  final BuildContext? pageContext;
  final List<String>? otherSpaceModels;
  final List<SpaceTemplateModel>? allSpaceModels;

  const AssignTagModelsDialog(
      {Key? key,
      required this.products,
      required this.subspaces,
      required this.addedProducts,
      required this.initialTags,
      this.onTagsAssigned,
      this.allTags,
      required this.spaceName,
      required this.title,
      this.pageContext,
      this.otherSpaceModels,
      this.spaceModel,
      this.allSpaceModels})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<String> locations = (subspaces ?? [])
        .map((subspace) => subspace.subspaceName)
        .toList()
      ..add('Main Space');

    return BlocProvider(
        create: (_) => AssignTagModelBloc()
          ..add(InitializeTagModels(
            initialTags: initialTags,
            addedProducts: addedProducts,
          )),
        child: BlocListener<AssignTagModelBloc, AssignTagModelState>(
          listener: (context, state) {},
          child: BlocBuilder<AssignTagModelBloc, AssignTagModelState>(
            builder: (context, state) {
              if (state is AssignTagModelLoaded) {
                final controllers = List.generate(
                  state.tags.length,
                  (index) => TextEditingController(text: state.tags[index].tag),
                );

                return AlertDialog(
                  title: Text(title),
                  backgroundColor: ColorsManager.whiteColors,
                  content: SingleChildScrollView(
                    child: Column(
                      children: [
                        ClipRRect(
                          borderRadius: BorderRadius.circular(20),
                          child: DataTable(
                            headingRowColor: WidgetStateProperty.all(
                                ColorsManager.dataHeaderGrey),
                            border: TableBorder.all(
                              color: ColorsManager.dataHeaderGrey,
                              width: 1,
                              borderRadius: BorderRadius.circular(20),
                            ),
                            columns: [
                              DataColumn(
                                  label: Text('#',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  label: Text('Device',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  numeric: false,
                                  label: Text('Tag',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  label: Text('Location',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                            ],
                            rows: state.tags.isEmpty
                                ? [
                                    DataRow(cells: [
                                      DataCell(
                                        Center(
                                          child: Text('No Devices Available',
                                              style: Theme.of(context)
                                                  .textTheme
                                                  .bodyMedium
                                                  ?.copyWith(
                                                    color: ColorsManager
                                                        .lightGrayColor,
                                                  )),
                                        ),
                                      ),
                                      const DataCell(SizedBox()),
                                      const DataCell(SizedBox()),
                                      const DataCell(SizedBox()),
                                    ])
                                  ]
                                : List.generate(state.tags.length, (index) {
                                    final tag = state.tags[index];
                                    final controller = controllers[index];
                                    final availableTags = getAvailableTags(
                                        allTags ?? [], state.tags, tag);

                                    return DataRow(
                                      cells: [
                                        DataCell(Text((index + 1).toString())),
                                        DataCell(
                                          Row(
                                            mainAxisAlignment:
                                                MainAxisAlignment.spaceBetween,
                                            children: [
                                              Expanded(
                                                  child: Text(
                                                tag.product?.name ?? 'Unknown',
                                                overflow: TextOverflow.ellipsis,
                                              )),
                                              const SizedBox(width: 10),
                                              Container(
                                                width: 20.0,
                                                height: 20.0,
                                                decoration: BoxDecoration(
                                                  shape: BoxShape.circle,
                                                  border: Border.all(
                                                    color: ColorsManager
                                                        .lightGrayColor,
                                                    width: 1.0,
                                                  ),
                                                ),
                                                child: IconButton(
                                                  icon: const Icon(
                                                    Icons.close,
                                                    color: ColorsManager
                                                        .lightGreyColor,
                                                    size: 16,
                                                  ),
                                                  onPressed: () {
                                                    context
                                                        .read<
                                                            AssignTagModelBloc>()
                                                        .add(DeleteTagModel(
                                                            tagToDelete: tag,
                                                            tags: state.tags));
                                                  },
                                                  tooltip: 'Delete Tag',
                                                  padding: EdgeInsets.zero,
                                                  constraints:
                                                      const BoxConstraints(),
                                                ),
                                              ),
                                            ],
                                          ),
                                        ),
                                        DataCell(
                                          Container(
                                            alignment: Alignment
                                                .centerLeft, // Align cell content to the left
                                            child: SizedBox(
                                              width: double
                                                  .infinity, // Ensure full width for dropdown
                                              child: DialogTextfieldDropdown(
                                                items: availableTags,
                                                initialValue: tag.tag,
                                                onSelected: (value) {
                                                  controller.text = value;
                                                  context
                                                      .read<
                                                          AssignTagModelBloc>()
                                                      .add(UpdateTag(
                                                        index: index,
                                                        tag: value,
                                                      ));
                                                },
                                              ),
                                            ),
                                          ),
                                        ),
                                        DataCell(
                                          SizedBox(
                                              width: double.infinity,
                                              child: DialogDropdown(
                                                items: locations,
                                                selectedValue: tag.location ??
                                                    'Main Space',
                                                onSelected: (value) {
                                                  context
                                                      .read<
                                                          AssignTagModelBloc>()
                                                      .add(UpdateLocation(
                                                        index: index,
                                                        location: value,
                                                      ));
                                                },
                                              )),
                                        ),
                                      ],
                                    );
                                  }),
                          ),
                        ),
                        if (state.errorMessage != null)
                          Text(state.errorMessage!,
                              style: Theme.of(context)
                                  .textTheme
                                  .bodySmall
                                  ?.copyWith(color: ColorsManager.warningRed)),
                      ],
                    ),
                  ),
                  actions: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceAround,
                      children: [
                        const SizedBox(width: 10),
                        Expanded(
                          child: Builder(
                            builder: (buttonContext) => CancelButton(
                              label: 'Add New Device',
                              onPressed: () async {
                                final updatedTags =
                                    List<TagModel>.from(state.tags);
                                final result =
                                    processTags(updatedTags, subspaces);

                                final processedTags =
                                    result['updatedTags'] as List<TagModel>;
                                final processedSubspaces = result['subspaces'];
                                if (context.mounted) {
                                  Navigator.of(context).pop();

                                  await showDialog<bool>(
                                    barrierDismissible: false,
                                    context: context,
                                    builder: (dialogContext) =>
                                        AddDeviceTypeModelWidget(
                                            products: products,
                                            subspaces: processedSubspaces,
                                            isCreate: false,
                                            initialSelectedProducts: TagHelper
                                                .createInitialSelectedProducts(
                                                    processedTags,
                                                    processedSubspaces),
                                            allTags: allTags,
                                            spaceName: spaceName,
                                            otherSpaceModels: otherSpaceModels,
                                            spaceTagModels: processedTags,
                                            pageContext: pageContext,
                                            spaceModel: SpaceTemplateModel(
                                                modelName: spaceName,
                                                tags: updatedTags,
                                                uuid: spaceModel?.uuid,
                                                internalId:
                                                    spaceModel?.internalId,
                                                subspaceModels:
                                                    processedSubspaces)),
                                  );
                                }
                              },
                            ),
                          ),
                        ),
                        const SizedBox(width: 10),
                        Expanded(
                          child: DefaultButton(
                            borderRadius: 10,
                            backgroundColor: state.isSaveEnabled
                                ? ColorsManager.secondaryColor
                                : ColorsManager.grayColor,
                            foregroundColor: ColorsManager.whiteColors,
                            onPressed: state.isSaveEnabled
                                ? () async {
                                    final updatedTags =
                                        List<TagModel>.from(state.tags);
                                    final result =
                                        processTags(updatedTags, subspaces);

                                    final processedTags =
                                        result['updatedTags'] as List<TagModel>;
                                    final processedSubspaces =
                                        result['subspaces']
                                            as List<SubspaceTemplateModel>;

                                    Navigator.of(context)
                                        .popUntil((route) => route.isFirst);

                                    await showDialog(
                                      context: context,
                                      builder: (BuildContext dialogContext) {
                                        return CreateSpaceModelDialog(
                                          products: products,
                                          allSpaceModels: allSpaceModels,
                                          allTags: allTags,
                                          pageContext: pageContext,
                                          otherSpaceModels: otherSpaceModels,
                                          spaceModel: SpaceTemplateModel(
                                              modelName: spaceName,
                                              tags: processedTags,
                                              uuid: spaceModel?.uuid,
                                              internalId:
                                                  spaceModel?.internalId,
                                              subspaceModels:
                                                  processedSubspaces),
                                        );
                                      },
                                    );
                                  }
                                : null,
                            child: const Text('Save'),
                          ),
                        ),
                        const SizedBox(width: 10),
                      ],
                    ),
                  ],
                );
              } else if (state is AssignTagModelLoading) {
                return const Center(child: CircularProgressIndicator());
              } else {
                return const Center(child: Text('Something went wrong.'));
              }
            },
          ),
        ));
  }

  List<String> getAvailableTags(
      List<String> allTags, List<TagModel> currentTags, TagModel currentTag) {
    return allTags
        .where((tagValue) => !currentTags
            .where((e) => e != currentTag) // Exclude the current row
            .map((e) => e.tag)
            .contains(tagValue))
        .toList();
  }

  int? checkTagExistInSubspace(
      TagModel tag, List<SubspaceTemplateModel>? subspaces) {
    if (subspaces == null) return null;
    for (int i = 0; i < subspaces.length; i++) {
      final subspace = subspaces[i];
      if (subspace.tags == null) continue;
      for (var t in subspace.tags!) {
        if (tag.internalId == t.internalId) {
          return i;
        }
      }
    }
    return null;
  }

  Map<String, dynamic> processTags(
      List<TagModel> updatedTags, List<SubspaceTemplateModel>? subspaces) {
    final modifiedTags = List<TagModel>.from(updatedTags);
    final modifiedSubspaces = List<SubspaceTemplateModel>.from(subspaces ?? []);

    if (subspaces != null) {
      for (var subspace in subspaces) {
        subspace.tags?.removeWhere(
          (tag) => !modifiedTags
              .any((updatedTag) => updatedTag.internalId == tag.internalId),
        );
      }
    }

    for (var tag in modifiedTags.toList()) {
      if (modifiedSubspaces.isEmpty) continue;

      final prevIndice = checkTagExistInSubspace(tag, modifiedSubspaces);

      if ((tag.location == 'Main Space' || tag.location == null) &&
          (prevIndice == null ||
              modifiedSubspaces[prevIndice].subspaceName == 'Main Space')) {
        continue;
      }

      if ((tag.location == 'Main Space' || tag.location == null) &&
          prevIndice != null) {
        modifiedSubspaces[prevIndice]
            .tags
            ?.removeWhere((t) => t.internalId == tag.internalId);
        continue;
      }

      if ((tag.location != 'Main Space' && tag.location != null) &&
          prevIndice == null) {
        final newIndex = modifiedSubspaces
            .indexWhere((subspace) => subspace.subspaceName == tag.location);
        if (newIndex != -1) {
          if (modifiedSubspaces[newIndex]
                  .tags
                  ?.any((t) => t.internalId == tag.internalId) !=
              true) {
            tag.location = modifiedSubspaces[newIndex].subspaceName;
            modifiedSubspaces[newIndex].tags?.add(tag);
          }
        }
        modifiedTags.removeWhere((t) => t.internalId == tag.internalId);
        continue;
      }

      if ((tag.location != 'Main Space' && tag.location != null) &&
          tag.location != modifiedSubspaces[prevIndice!].subspaceName) {
        modifiedSubspaces[prevIndice]
            .tags
            ?.removeWhere((t) => t.internalId == tag.internalId);
        final newIndex = modifiedSubspaces
            .indexWhere((subspace) => subspace.subspaceName == tag.location);
        if (newIndex != -1) {
          if (modifiedSubspaces[newIndex]
                  .tags
                  ?.any((t) => t.internalId == tag.internalId) !=
              true) {
            tag.location = modifiedSubspaces[newIndex].subspaceName;
            modifiedSubspaces[newIndex].tags?.add(tag);
          }
        }

        modifiedTags.removeWhere((t) => t.internalId == tag.internalId);
        continue;
      }

      if ((tag.location != 'Main Space' && tag.location != null) &&
          tag.location == modifiedSubspaces[prevIndice!].subspaceName) {
        modifiedTags.removeWhere((t) => t.internalId == tag.internalId);
        continue;
      }

      if ((tag.location == 'Main Space' || tag.location == null) &&
          prevIndice != null) {
        modifiedSubspaces[prevIndice]
            .tags
            ?.removeWhere((t) => t.internalId == tag.internalId);
      }
    }

    return {
      'updatedTags': modifiedTags,
      'subspaces': modifiedSubspaces,
    };
  }
}
